@@ -1,12 +1,25 @@
-;;; engine-mode.el --- Define and query search engines from within Emacs.
+;;; engine-mode.el --- Define and query search engines from within Emacs
 
 ;; Author: Harry R. Schwartz <hello@harryrschwartz.com>
-;; Version: 2.1.1
+;; Version: 2.2.1
 ;; URL: https://github.com/hrs/engine-mode
 ;; Package-Requires: ((cl-lib "0.5"))
 
 ;; This file is NOT part of GNU Emacs.
 
+;; This program is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
+
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 ;;; Commentary:
 
 ;; `engine-mode' is a global minor mode for Emacs. It enables you to
@@ -33,28 +46,18 @@
 ;; `C-x / d' is now bound to the new function
 ;; engine/search-duckduckgo'! Nifty.
 
-;;; License:
-
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation, either version 3 of the License, or
-;; (at your option) any later version.
-
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 ;;; Code:
-(eval-when-compile (require 'cl-macs))
+(eval-when-compile (require 'cl-lib))
+(require 'format-spec)
+
+(defgroup engine-mode nil
+  "Define search engines, bind them to keybindings, and query them."
+  :group 'external)
 
 (defcustom engine/keybinding-prefix "C-x /"
   "The default engine-mode keybindings prefix."
-  :group 'engine-mode
-  :type 'string)
+  :type '(choice (string :tag "Key")
+                 (const :tag "No keybinding" nil)))
 
 (define-prefix-command 'engine-mode-prefixed-map)
 (defvar engine-mode-prefixed-map)
@@ -78,12 +81,13 @@
 For example, to use \"C-c s\" instead of the default \"C-x /\":
 
 \(engine/set-keymap-prefix (kbd \"C-c s\"))"
-  (define-key engine-mode-map (kbd engine/keybinding-prefix) nil)
+  (when engine/keybinding-prefix
+    (define-key engine-mode-map (kbd engine/keybinding-prefix) nil))
   (define-key engine-mode-map prefix-key engine-mode-prefixed-map))
 
-(defcustom engine/browser-function nil
+(defcustom engine/browser-function browse-url-browser-function
   "The default browser function used when opening a URL in an engine.
-Defaults to `nil' which means to go with `browse-url-browser-function'."
+Defaults to `browse-url-browser-function'."
   :group 'engine-mode
   :type 'symbol)
 
@@ -106,8 +110,7 @@ Defaults to `nil' which means to go with `browse-url-browser-function'."
 (defun engine/execute-search (search-engine-url browser-function search-term)
   "Display the results of the query."
   (interactive)
-  (let ((browse-url-browser-function (or browser-function
-                                         browse-url-browser-function)))
+  (let ((browse-url-browser-function browser-function))
     (browse-url
      (format-spec search-engine-url
                   (format-spec-make ?s (url-hexify-string search-term))))))
@@ -116,9 +119,8 @@ Defaults to `nil' which means to go with `browse-url-browser-function'."
   (intern (concat "engine/search-" (downcase (symbol-name engine-name)))))
 
 (defun engine/docstring (engine-name)
-  (concat "Search "
-          (capitalize (symbol-name engine-name))
-          " for the selected text. Prompt for input if none is selected."))
+  (format "Search %s for the selected text.\nPrompt for input if none is selected."
+          (capitalize (symbol-name engine-name))))
 
 (defun engine/bind-key (engine-name keybinding)
   (when keybinding
@@ -144,7 +146,7 @@ to always upcase our search terms, we might use:
 
 \(defengine duckduckgo
   \"https://duckduckgo.com/?q=%s\"
-  :term-transformation-hook 'upcase)
+  :term-transformation-hook upcase)
 
 In this case, searching for \"foobar\" will hit the url
 \"https://duckduckgo.com/?q=FOOBAR\".
@@ -157,13 +159,14 @@ Keybindings are in the `engine-mode-map', so they're prefixed.
 For example, to search Wikipedia, use:
 
   (defengine wikipedia
-    \"http://www.wikipedia.org/search-redirect.php?language=en&go=Go&search=%s\"
+    \"https://www.wikipedia.org/search-redirect.php?language=en&go=Go&search=%s\"
     :keybinding \"w\"
     :docstring \"Search Wikipedia!\")
 
 Hitting \"C-x / w\" will be bound to the newly-defined
 `engine/search-wikipedia' function."
 
+  (declare (indent 1))
   (cl-assert (symbolp engine-name))
   `(prog1
      (defun ,(engine/function-name engine-name) (search-term)
